local res = ...
local redis_redirect_db = 1
local php_article_create_path = "/internal/create-article.php"

local url = ngx.var.uri
local host = ngx.var.http_host

-- Config properties
local redis_found_ttl = 86400 -- 1day
local redis_notfound_ttl = 600 -- 10min

local red = ledge:ctx().redis
red:select(redis_redirect_db)

local json = require 'cjson'
local http = require 'resty.http'
local config = require '/utils/config'
local global_config = config['global']

local function get_original_domain(requested_host)
    local domain = requested_host
    local sites = config['sites']

    -- copy all the attributes from t
    for k, data in pairs(sites) do
        -- If match on production
        if requested_host == sites or
                requested_host == data['subdomain'] .. '.' .. global_config['stage_domain'] or
                requested_host == data['subdomain'] .. '.' .. global_config['dev_domain'] then
            domain = data['original_domain']
        end
    end
    return domain
end

-- Prepare url for matching it in Open Content
local function prepare_url(url)

    -- Remove first char if slash
    if string.sub(url, 0, 1) == '/' then
        url = string.sub(url, 2)
    end

    -- Remove last char if slash
    if string.sub(url, -1) == '/' then
        url = string.sub(url, 0, string.len(url) - 1)
    end

    url = string.format("%s/%s", get_original_domain(host), url)
    return url
end

-- Set content to Redis, set "value" for "key" with "ttl" as expire (seconds).
local function redis_set(key, value, ttl)
    local ok, err = red:setex(key, ttl, value)
    if not ok then
        ngx.log(ngx.STDERR, err)
        return nil
    end
end

-- Get content from redis with given key.
local function redis_get(key)
    local res, err = red:get(key)
    if not res then
        return ngx.null
    end

    return res
end

-- Validates data recieved from Open Content.
local function validate_oc_data(data)
    local obj, pos, err = json.decode(data)

    if err then
        ngx.say("Error parsing JSON data from OC: ", err)
    else
        if obj.hits.hits[1] ~= nil then
            return obj.hits.hits[1].id
        end
    end

    return nil
end

-- Makes request to Wordpress and creates an article object, returns the url of the new article.
local function http_wp_create_article(uuid, host)
    local httpc = http.new()
    local ok, err = httpc:connect('127.0.0.1', 80)
    local client_body_reader, err = httpc:get_client_body_reader()

    local res, err = httpc:request({
        method = ngx.req.get_method(),
        path = php_article_create_path,
        query = { uuid = uuid, host = host },
        body = client_body_reader
    })

    if not res then
        ngx.log(ngx.STDERR, string.format("Could not create article from uuid: %s", uuid), err)
        return nil
    end

    if res.status >= 200 and res.status < 300 then
        return res:read_body()
    else
        ngx.log(ngx.STDERR, string.format("Article create returned HTTP code that's not success: " .. res.status))
    end

    return nil
end

local oc_config = config['oc']

local oc_url = oc_config['url']
local oc_port = oc_config['port']
local oc_user = oc_config['user']
local oc_pass = oc_config['password']

if url ~= nil then
    -- Check if key already exists in redis
    local key = "redirect_url_" .. url
    local redirect_url = redis_get(key)

    -- Return nil if no redirect_url was found last time
    if redirect_url == '0' then
        return nil
    end

    if redirect_url ~= ngx.null then
        return ngx.redirect(redirect_url, ngx.HTTP_MOVED_PERMANENTLY)
    end

    local prepared_url = prepare_url(url)
    local oc_query = string.format('contenttype:Article AND (OriginalUrls:"http://%s" OR OriginalUrls:"https://%s")', prepared_url, prepared_url);

    local httpc = http.new()
    local ok, err = httpc:connect(oc_url, oc_port)
    local client_body_reader, err = httpc:get_client_body_reader()

    local path = string.format('/opencontent/search?q=%s', ngx.escape_uri(oc_query));
    local req_params = {
        path = path,
        headers = { Authorization = "Basic " .. ngx.encode_base64(oc_user .. ":" .. oc_pass) },
        method = ngx.req.get_method(),
        body = client_body_reader
    }

    local origin, e = httpc:request(req_params)

    if origin.has_body then
        local oc_data = origin:read_body()
        local uuid = validate_oc_data(oc_data)

        if uuid ~= nil then
            -- Send request to PHP and create the article and get the new permalink back.
            local permalink = http_wp_create_article(uuid, host)
            if permalink ~= nil then
                redis_set(key, permalink, redis_found_ttl)
                return ngx.redirect(permalink, ngx.HTTP_MOVED_PERMANENTLY)
            end
        end
    end

    redis_set(key, '0', redis_notfound_ttl)
end

return nil
