local TuloAuth = {}
TuloAuth.__index = TuloAuth

function TuloAuth.new()

    local self = setmetatable({}, TuloAuth)

    self.http = require 'resty.http'
    self.json = require 'cjson'
    self.config = require '/tulo-auth/config'

    -- Site config variables
    self.site_config = self.config[self:get_config_key()]
    self.base_tulo_url = self.site_config.sso.url
    self.client_id = self.site_config.sso.client
    self.client_secret = self.site_config.sso.secret
    self.redirect_uri = self.site_config.sso.forward_url

    self:set_form_data()
    self.access_token = self:get_access_token('/oauth2/login/x')
    self.external_access_token = self:get_access_token('/external/account/w')

    return self
end

-- GET CONFIG KEY
function TuloAuth.get_config_key()
    local config_key = string.gsub(ngx.var.http_host, 'www%.', '')
    return config_key
end

-- SET FROM DATA
function TuloAuth.set_form_data(self)
    ngx.req.read_body()
    local post_args, err = ngx.req.get_post_args()
    if not post_args then
        return ngx.redirect(ngx.var.scheme .. '://' .. ngx.var.http_host .. '/?tulo_error=unknown_error', ngx.HTTP_MOVED_TEMPORARILY)
    end

    if post_args['email'] ~= nil and post_args['password'] ~= nil and post_args['forward_url'] ~= nil then
        self.username = post_args['email']
        self.password = post_args['password']
        self.forward_url = post_args['forward_url']
        self.persist = 'false'
        if post_args['rememberMe'] ~= nil and post_args['rememberMe'] == 'remember-me' then
            self.persist = 'true'
        end
    else
        return ngx.redirect(ngx.var.scheme .. '://' .. ngx.var.http_host .. '/?tulo_error=unknown_error', ngx.HTTP_MOVED_TEMPORARILY)
    end
end

-- GET ACCESS TOKEN
function TuloAuth.get_access_token(self, scope)
    local httpc = self.http.new()
    local tulo_error = 'unknown'

    local res, err = httpc:request_uri('https://' .. self.base_tulo_url .. '/api/authorization/access_token', {
        method = "POST",
        body = 'client_id=' .. self.client_id .. '&client_secret=' .. self.client_secret .. '&grant_type=none&redirect_uri=&scope=' .. scope,
        ssl_verify = false,
        headers = {
            ['Content-Type'] = 'application/x-www-form-urlencoded',
            ['Accept'] = 'application/json'
        }
    })

    if not res then
        ngx.log(ngx.STDERR, string.format('Tulo access token for %s got no response: ', self.forward_url), err)
        return ngx.redirect(self.forward_url .. '?tulo_access_token_error=nores', ngx.HTTP_MOVED_TEMPORARILY)
    end
    local body = self.json.decode(res.body)

    if res.status >= 200 and res.status < 300 then
        return body['access_token']
    end

    ngx.log(ngx.STDERR, string.format('Tulo access token  - responded with code "%s": %s', res.status, res.body))

    -- Set error from tulo response
    if body['error'] then
        tulo_error = body['error']
    end

    return ngx.redirect(string.format('%s?tulo_access_token_error=%s', self.forward_url, tulo_error), ngx.HTTP_MOVED_TEMPORARILY)
end

-- CREATE USER
function TuloAuth.create_user(self)

    local httpc = self.http.new()
    local res, err = httpc:request_uri('https://' .. self.base_tulo_url .. '/external/api/v1/accounts', {
        method = "POST",
        body = self.json.encode({ email = self.username, password = self.password }),
        ssl_verify = false,
        headers = {
            ['Content-Type'] = 'application/json',
            ['Accept'] = 'application/json',
            ['Authorization'] = 'Bearer ' .. self.external_access_token
        }
    })

    if not res then
        return nil
    end

    if res.status >= 200 and res.status < 300 then
        return res.body
    end

    local error = self.json.decode(res.body)
    local error_msg = 'unknown_error'

    if error['code'] then
        if error['code'] == 'already_exists' and error['field'] == 'email' then
            error_msg = 'email_already_exists'
        elseif error['code'] == 'invalid_parameter' and error['field'] == 'password' then
            error_msg = 'password_too_short'
        elseif error['code'] == 'invalid_parameter' and error['field'] == 'email' then
            error_msg = 'invalid_email'
        end
    end

    return ngx.redirect(self.forward_url .. '?tulo_create_error=' .. error_msg, ngx.HTTP_MOVED_TEMPORARILY)
end

-- LOGIN USER
function TuloAuth.login_user(self)

    local httpc = self.http.new()
    local res, err = httpc:request_uri('https://' .. self.base_tulo_url .. '/oauth2/login', {
        method = "POST",
        body = 'username=' .. self.username .. '&password=' .. ngx.escape_uri(self.password) .. '&persist=' .. self.persist,
        ssl_verify = false,
        headers = {
            ['Content-Type'] = 'application/x-www-form-urlencoded',
            ['Accept'] = 'application/json',
            ['Authorization'] = 'OAuth ' .. self.access_token
        }
    })

    if not res then
        return nil
    end

    if res.status >= 200 and res.status < 300 then
        return self.json.decode(res.body)
    end

    return nil
end

-- HANDLE LOGIN DATA
function TuloAuth.handle_login_data(self, login_data)

    local login_status = 'error'
    if login_data ~= nil then
        login_status = login_data['status']
    end

    if login_status == 'error' then

        local error_param = 'unknown_error_login'
        if login_data ~= nil and login_data['code'] ~= nil and login_data['code'] == 'invalid_username_or_password' then
            error_param = login_data['code']
        end

        return ngx.redirect(self.forward_url .. '?tulo_error=' .. error_param, ngx.HTTP_MOVED_TEMPORARILY)
    elseif login_status == 'ok' then
        local login_handle = login_data['result']

        self:redirect_user(login_handle)
    end

    return ngx.redirect(ngx.var.scheme .. '://' .. ngx.var.http_host .. '/?tulo_error=unknown_error', ngx.HTTP_MOVED_TEMPORARILY)
end

-- REDIRECT USER
function TuloAuth.redirect_user(self, login_handle)
    local state = ngx.escape_uri(self.forward_url)
    local redirect_uri = ngx.escape_uri(self.redirect_uri)

    local req_params = '?client_id=' .. ngx.escape_uri(self.client_id) .. '&redirect_uri=' .. redirect_uri .. '&state=' .. state .. '&response_type=code&oid=&scope=&login_handle=' .. ngx.escape_uri(login_handle)
    local redirect_url = 'https://' .. self.base_tulo_url .. '/oauth2/auth' .. req_params

    return ngx.redirect(redirect_url, ngx.HTTP_MOVED_TEMPORARILY)
end

return TuloAuth
